/*!
 * \file MEMM.cpp
 * \brief MEMM abstract class implementation
 * \author Jean-Francois REY
 * \version 1.0
 * \date 23 Sept 2014
 */


#include "MEMM.hpp"

MEMM::MEMM() : dist_norm(0,1.0), unif(0,1.0), generator(static_cast<unsigned int>(std::time(0))), gauss(generator,dist_norm), accept(generator,unif)
{
  NPar = 0;
  Ns = 0;
  ParamFec = IndivFec = ParamDisp = NULL;
  pLoi = NULL;
  useMatError = false;
  NMistypeMax=1000;
}

MEMM::~MEMM()
{
  
  if(ParamFec)
  {
    ParamFec->close();
    delete ParamFec;
  }
  if(IndivFec)
  {
    IndivFec->close();
    delete IndivFec;
  }
  if(ParamDisp)
  {
    ParamDisp->close();
    delete ParamDisp;
  }
  if(pLoi) delete pLoi;

  fatherID.clear();
  
  allParents.erase(allParents.begin(),allParents.end());
  allParents.clear();

  allSeeds.erase(allSeeds.begin(),allSeeds.end());
  allSeeds.clear();

}

void MEMM::init(Parameters * p)
{

    if(p == NULL) {
      cerr << "Error Parameters in init function" << endl;
      return;
    }
 
    char *temp;
    temp = p->getValue(MEMM_POSTERIOR_TYPE);
    if( strcmp("assignment",temp) == 0 ) posterior=1;
    else if( strcmp("seedpool",temp) == 0 ) posterior=2;
    else posterior=NULL;
    cout<<"\t-"<<temp<<" "<< posterior<<endl;
    

    
  // Load parents from file and fatherID
  loadParentFile(p);

  // Load seeds 
  loadSeeds(p);

  // Calcul distances
  calculDistances(p);

  // load allelic frequencies
  loadAllelicFreq(p);

  // calcul transition matrix
  calculTransitionMatrix(p);  

  calculWeightPollenDonor();

    loadPrior(p);
    loadParameters(p);
    


  // TODO move to new function endRun() 
  //for (int k=0; k<NPar; k++) {delete allParents[k];}
  //for (int k=0; k<Ns; k++) {delete allSeeds[k];}
}

void MEMM::burnin()
{
    cout << "Burn in "<<Nburn<<" iterations ..." <<endl;
    if(Nburn) mcmc(Nburn,0);
    cout << "Burn in iterations : OK" <<endl<<endl;

}

void MEMM::run()
{
    cout << "Run " << Nstep <<" iterations" << endl;
    if(Nstep) mcmc(Nstep,Nthin);
    cout << "MCMC finished " << Nstep << " iterations"
    << endl << "You can visualize the results from R"<<endl;
}



/***************************** PROTECTED ***********************************/

void MEMM::loadParentFile(Parameters * p)
{
  //////////////////////////////////
  //    LOADING PARENTS           //
  //////////////////////////////////

  string name;
  char * temp;

  if(p == NULL) return;

  if(allParents.size() != 0) allParents.clear();

  cout<<endl<<"# Loading parents..."<<endl;
  temp = p->getValue(MEMM_PARENTS_FILE_NAME);
  ifstream parents(temp);
  if(temp) delete []temp;
  temp = p->getValue(MEMM_PARENTS_FILE_NUMBER_OF_LOCUS);
  Nl = atoi(temp);
  if(temp) delete []temp;
  temp = p->getValue(MEMM_PARENTS_FILE_CLASS_COV);
  Nql = atoi(temp);
  if(temp) delete []temp;
  temp = p->getValue(MEMM_PARENTS_FILE_QT_COV);
  Nqt = atoi(temp);
  if(temp) delete []temp;
  temp = p->getValue(MEMM_PARENTS_FILE_WEIGHT_VAR);
  Nw = atoi(temp);
  if(temp) delete []temp;

  NPar = 0;
  if(parents.good())
  {
    while(parents >> name)
    {
      cout<<name<<"...";
      allParents.push_back(new parent(name,parents,Nql,Nqt,Nl));
      NPar++;
    }
    for(int k=0; k<NPar; k++)
    {
      fatherID[(*allParents[k]).getName()]=k;
    }
    cout<<endl<< NPar << " parents loaded. " << std::endl;
  }
  else
  {
    cerr<<"#ERROR ! Can not read parents file !"<<endl<<"Continue..."<<endl;
  }
  parents.close();

}

void MEMM::loadSeeds(Parameters * p)
{

  int Nsl;
  string name;
  char * temp, *filename;

  if(p == NULL)return;

  cout<<endl<<"# Loading seeds..."<<endl;
  if(allSeeds.size() != 0) allSeeds.clear();

  filename = p->getValue(MEMM_OFFSPRING_FILE_NAME);
  ifstream seeds(filename);
  temp = p->getValue(MEMM_OFFSPRING_FILE_NUMBERS_OF_LOCUS);
  Nsl = atoi(temp);
  if(temp) delete []temp;

  Ns = 0;
  if(seeds.good())
  {
    while(seeds >> name)
    {
      cout<<name<<"...";
      allSeeds.push_back(new graine(name,seeds,Nsl));
      Ns++;
    }
    cout<<endl<<Ns<<" seeds loaded."<<endl;
    seeds.close();
  }
  else
  {
    cerr<<"#ERROR ! Can not read "<<filename<<" seeds file !"<<endl<<"Continue..."<<endl;
  }
  if(filename) delete []filename;

}

void MEMM::calculDistances(Parameters *p)
{
  //cerr<<"# Call of Default function MEMM::CalculDistances()\n  this fuction is not implemented"<<endl;
  if(allParents.size() != NPar || NPar == 0)
  {
    cerr<<"# ERROR : CalculDistance, number of parents incorrect"<<endl;
    return;
  }

    ofstream exportDistMP;
    exportDistMP.open("exportDistMP.txt");

  for(int i=DistMP.size()-1; i>=0 ; i--) DistMP[i].clear();
  for(int i=DistMP.size()-1; i>=0 ; i--) AzimMP[i].clear();
  DistMP.clear();
  DistMP.resize(NPar,vector<double>(NPar,0.));
    AzimMP.clear();
    AzimMP.resize(NPar,vector<double>(NPar,0.));

  cout << endl << "# Computing distances and azimuths from x-y coordinates of parents...";
  for (int m=0; m<NPar; m++)
      for (int p=0; p<NPar; p++) {
      DistMP[m][p]=(*allParents[m]).dist(*allParents[p]);
      AzimMP[m][p]=(*allParents[m]).azim(*allParents[p]);
          if (m != p) exportDistMP << DistMP[m][p] << "\t" << AzimMP[m][p] << endl;

          if (m<=2 && p<=2) {cout << m << " x " << p << " | " << DistMP[m][p] << " , " << AzimMP[m][p] << endl;}

      }
    cout<<endl << "Computing distances and azimuths : OK. " << endl;
    exportDistMP.close();

}

void MEMM::loadAllelicFreq(Parameters * p)
{

  bool readfile = false;
  char * temp;
  int ntemp;
  double floattemp, stemp;//,a,ab;
  vector< std::vector<int> > AllSize (Nl);
  vector< std::map<int,double> > SizeFreq (Nl);
    
    
  Na.clear();
  Na.assign(Nl, 0);

  for(int i=AllFreq.size()-1; i>=0 ; i--) AllFreq[i].clear();
  AllFreq.clear();
  AllFreq.resize(Nl);

  for(int i=SizeAll.size()-1; i>=0 ; i--) SizeAll[i].clear();
  SizeAll.clear();
  SizeAll.resize(Nl);

  temp = p->getValue(MEMM_AF_FILE_MODE);
  if( strcmp("file",temp) == 0)
  {
    delete []temp;
    temp = p->getValue(MEMM_AF_FILE_NAME);
    ifstream fall(temp);
    if(fall.good())
    {
      cout << endl<<"# Loading allelic frequencies from file " <<temp<<" ..."<<endl;
      if(temp) delete []temp;
      temp = NULL;
      for (int l=0; l<Nl; l++)
      {
        fall >> Na[l];
        stemp=0;
        for (int a=0; a<Na[l]; a++)
        {
          fall >> ntemp >> floattemp;
          stemp+=floattemp;
          AllSize[l].push_back(ntemp);
          AllFreq[l].push_back(floattemp);
          SizeAll[l][ntemp]=a;
        }
        cout << "Locus " << l << " : " << Na[l] << " alleles" <<endl;
        for (int a=0; a<Na[l]; a++)
        {
          AllFreq[l][a] /= stemp;
          SizeFreq[l][AllSize[l][a]]=AllFreq[l][a];
          std::cout << AllSize[l][a] << ":" << AllFreq[l][a] << " | ";
        }
        cout << endl;
      }
      cout << "Loading of allelic frequencies : OK. " <<endl;
      fall.close();
      readfile = true;
    }
    else{
      readfile = false;
      cerr<<"ERROR can't read allelic frequencies from "<<temp<<endl;
    }
  }
  
  if(!readfile)
  {
    if(temp) delete []temp;
    temp = NULL;
    int all;
    std::cout << endl << "# Computing allelic frequencies from adult trees..." << std::endl;
    for (int l=0; l<Nl; l++)
    {
      ntemp=0;
      for (int k=0; k<NPar; k++)
      {
        for (int m=0; m<2; m++)
        {
          all = (*allParents[k]).getGeno(l)._allele[m];
          if (all>-1)
          {
            if ( (SizeAll[l].find(all)) == SizeAll[l].end() )
            {
              Na[l]++;
              ntemp++;
              std::cout << Na[l] << "...";
              AllSize[l].push_back(all);
              AllFreq[l].push_back(1.);
              SizeAll[l][all]=Na[l]-1;
            }
            else {
              ntemp++;
              AllFreq[l][SizeAll[l][all]]++;
            }
          }
        }
      }
      std::cout << "Locus " << l << " : " << Na[l] << " alleles (" << ntemp << ")" <<std::endl;

      for (int a=0; a<Na[l]; a++)
      {
        AllFreq[l][a] /= ntemp;
        SizeFreq[l][AllSize[l][a]]=AllFreq[l][a];
        cout << AllSize[l][a] << ":" << AllFreq[l][a] << " | ";
      }
      cout << endl;
    }
    std::cout << "Estimation of allelic frequencies : OK. " <<std::endl << std::endl;
  }

    
  if(temp) delete []temp;

    loadMatrixError(p,AllSize);


}

void MEMM::loadMatrixError(Parameters *p, vector< vector<int> > & AllSize)
{
  char * temp;
  vector<double> motif(Nl);
  vector<double> eps0(Nl);
  vector<double> epsInf(Nl);

  temp = p->getValue(MEMM_LOCUS_ERROR);
  if(temp && strcmp("",temp)!=0)
  {
    std::cout<<"# Will use Locus error matrix"<<std::endl<<std::endl;
    delete[] temp;
      
    temp = p->getValue(MEMM_LOCUS_ERROR_FILE);
    ifstream inputLocusError(temp);
   
      
    if(inputLocusError.good())
    {
      cout << endl<<"# Loading motif, eps0 and epsInf from file " <<temp<<" ..."<<endl;
      useMatError = true;
        if(temp) delete[] temp;
        temp = p->getValue(MEMM_LOCUS_ERROR_NMISTYPEMAX);
        NMistypeMax = atoi(temp);
        if(temp) delete[] temp;
  
      int l;
      for(l=0; l<Nl && inputLocusError.good() ; l++)
      {
        inputLocusError >> motif[l];
        inputLocusError >> eps0[l];
        inputLocusError >> epsInf[l];
      }
      if(l != Nl && l>0)
        for(l=1; l<Nl ; l++)
        {
          motif[l] =motif[0];
          eps0[l] = eps0[0];
          epsInf[l] = epsInf[0];
        }
      inputLocusError.close();

      MatError.resize(Nl);
      for (int l=0; l<Nl; l++)
      {
        MatError[l].resize(Na[l]);
        for (int k=0; k<Na[l]; k++){ MatError[l][k].resize(Na[l]); }
        for (int j=0; j<Na[l]; j++)
        {
          for (int k=0; k<Na[l]; k++)
          {
            if ( AllSize[l][j]==AllSize[l][k] ){ MatError[l][k][j]=1-eps0[l]-epsInf[l]; }
              else if ( abs(AllSize[l][j]-AllSize[l][k])==motif[l] ){ MatError[l][k][j]=eps0[l]/2+epsInf[l]/(Na[l]-1); }
              else{ MatError[l][k][j]=epsInf[l]/(Na[l]-1); }
          }
        }
      }
        
     //Essai ecriture... Pour debuggage....
     //   cout << AllSize[1][0] << "  " << AllSize[1][1] << "  " << motif[1] << "  " <<endl;
     //   for (int j=0; j<Na[1]; j++)
     //       {for (int k=0; k<Na[1]; k++) cout << MatError[1][k][j] << "  ";
     //           cout<<endl;}
        
 
      motif.clear();
      eps0.clear();
      epsInf.clear();
    }
    else{
      std::cerr<<"ERROR: can't open File "<<temp<<std::endl;
      useMatError = false;
      if(temp) delete[] temp;
    }
  }

  else{
    useMatError = false;
    if(temp) delete[] temp;
  }
}

void MEMM::calculTransitionMatrix(Parameters *p)
{

  cerr<<endl<<"# Call of Default function MEMM::CalculTransitionMatrix()\n  this fuction is not implemented"<<endl;

}

void MEMM::loadPrior(Parameters *p)
{
  char * temp, * temp2, *temp3;

  cout << endl << "Reading prior..."<<endl;
  temp = p->getValue(MEMM_GAMA_INIT);
  GamA = atof(temp);
  temp2 = p->getValue(MEMM_GAMA_MIN);
  mGamA = atof(temp2);
  temp3 = p->getValue(MEMM_GAMA_MAX); 
  MGamA = atof(temp3);
  cout << "GamA : "<< GamA << " [" <<mGamA <<"|"<< MGamA << "]"<<endl;
  if(temp) delete []temp;
  if(temp2) delete []temp2;
  if(temp3) delete []temp3;

    
        // pi0
    temp = p->getValue(MEMM_IND_FEC_DIST);
    if( strcmp("zigamma",temp) == 0 ) {
        if(temp) delete []temp;
        temp = p->getValue(MEMM_PI0_INIT);
        Pi0 = atof(temp);
        temp2 = p->getValue(MEMM_PI0_MIN);
        mPi0 = atof(temp2);
        temp3 = p->getValue(MEMM_PI0_MAX);
        MPi0 = atof(temp3);
        cout << "pi0s : "<< Pi0 << " [" << mPi0 <<"|"<< MPi0 << "]"<<endl;
        if(temp) delete []temp;
        if(temp2) delete []temp2;
        if(temp3) delete []temp3;
    }else { if(temp) delete []temp; }
    
  temp = p->getValue(MEMM_DELTA_INIT);
  Scale = atof(temp);
  temp2 = p->getValue(MEMM_DELTA_MIN);
  mScale = atof(temp2);
  temp3 = p->getValue(MEMM_DELTA_MAX);
  MScale = atof(temp3);
  cout << "Scale : "<< Scale << " [" <<mScale <<"|"<< MScale << "]"<<endl;
  if(temp) delete []temp;
  if(temp2) delete []temp2;
  if(temp3) delete []temp3;

  temp = p->getValue(MEMM_SHAPE_B_INIT);
  Shape = atof(temp);
  temp2 = p->getValue(MEMM_SHAPE_B_MIN);
  mShape = atof(temp2);
  temp3 = p->getValue(MEMM_SHAPE_B_MAX);
  MShape = atof(temp3);
  cout << "Shape : "<< Shape << " [" <<mShape <<"|"<< MShape << "]"<<endl;
  if(temp) delete []temp;
  if(temp2) delete []temp2;
  if(temp3) delete []temp3;

    // NEW anisotropie
    
    temp = p->getValue(MEMM_THETA_INIT);
    Theta = atof(temp);
    temp2 = p->getValue(MEMM_THETA_MIN);
    mTheta = atof(temp2);
    temp3 = p->getValue(MEMM_THETA_MAX);
    MTheta = atof(temp3);
    cout << "Theta : "<< Theta << " [" <<mTheta <<"|"<< MTheta << "]"<<endl;
    if(temp) delete []temp;
    if(temp2) delete []temp2;
    if(temp3) delete []temp3;
    
    temp = p->getValue(MEMM_KAPPA_INIT);
    Kappa = atof(temp);
    temp2 = p->getValue(MEMM_KAPPA_MIN);
    mKappa = atof(temp2);
    temp3 = p->getValue(MEMM_KAPPA_MAX);
    MKappa = atof(temp3);
    cout << "Kappa : "<< Kappa << " [" <<mKappa <<"|"<< MKappa << "]"<<endl;
    if(temp) delete []temp;
    if(temp2) delete []temp2;
    if(temp3) delete []temp3;

  temp = p->getValue(MEMM_MIG_RATE_M_INIT);
  Mig = atof(temp);
  temp2 = p->getValue(MEMM_MIG_RATE_M_MIN);
  mMig = atof(temp2);
  temp3 = p->getValue(MEMM_MIG_RATE_M_MAX);
  MMig = atof(temp3);
  cout << "Mig : "<< Mig << " [" <<mMig <<"|"<< MMig << "]"<<endl;
  if(temp) delete []temp;
  if(temp2) delete []temp2;
  if(temp3) delete []temp3;

  temp = p->getValue(MEMM_SELF_RATE_S_INIT);
  Self = atof(temp);
  temp2 = p->getValue(MEMM_SELF_RATE_S_MIN);
  mSelf = atof(temp2);
  temp3 = p->getValue(MEMM_SELF_RATE_S_MAX);
  MSelf = atof(temp3);
  cout << "Self : "<< Self << " [" <<mSelf <<"|"<< MSelf << "]"<<endl;
  if(temp) delete []temp;
  if(temp2) delete []temp2;
  if(temp3) delete []temp3;
  cout<<"...end reading prior"<<endl;
}


void MEMM::loadParameters(Parameters *p)
{
  char * temp;
  cout << endl << "Load parameters..."<<endl;

  temp = p->getValue(MEMM_SEED);
  if(atoi(temp) >= 0)
  {
    cout << "New seed : "<<temp<<endl;
    generator.seed(static_cast<long unsigned int>(atoi(temp)));
  }
  else {
      cout << "New seed : generated from time "<<temp<<endl;
      generator.seed(static_cast<long unsigned int>(time(NULL)));
  }

  if(temp) delete []temp;

  temp = p->getValue(MEMM_IND_FEC_DIST);
  if( strcmp("LN",temp) == 0 ) pLoi = new MEMM_logNormal(gauss);
  else if( strcmp("gamma",temp) == 0 || strcmp("gama",temp) == 0) pLoi = new MEMM_gamma(GamA,accept);
    else pLoi = new MEMM_zigamma(Pi0, GamA, accept);
  if(temp) delete []temp;

  temp = p->getValue(MEMM_BURNIN);
  Nburn = atoi(temp);
  if(temp) delete []temp;
  temp = p->getValue(MEMM_ITE);
  Nstep = atoi(temp);
  if(temp) delete []temp;
  temp = p->getValue(MEMM_THIN);
  Nthin = atoi(temp);
  if(temp) delete []temp;
  cout<<endl<< "Burnin "<< Nburn << " | Iteration "<<Nstep<<" | Thin "<<Nthin<<endl;
  

  cout<<"Ouput files :"<<endl;
  temp = p->getValue(MEMM_GAMA_FILE_NAME);
  ParamFec = new std::ofstream(temp, ofstream::out);
  if(ParamFec->good()) cout<<"\t-"<<temp<<endl;
  else cerr<<"#ERROR openning "<<temp<<" file"<<endl;
  if(temp) delete []temp;

  temp = p->getValue(MEMM_IND_FEC_FILE_NAME);
  IndivFec = new std::ofstream(temp, ofstream::out);
  if(IndivFec->good()) cout<<"\t-"<<temp<<endl;
  else cerr<<"#ERROR openning "<<temp<<" file"<<endl;
  if(temp) delete []temp;

  temp = p->getValue(MEMM_DISP_FILE_NAME);
  ParamDisp = new std::ofstream(temp, ofstream::out);
  if(ParamDisp->good()) cout<<"\t-"<<temp<<endl;
  else cerr<<"#ERROR openning "<<temp<<" file"<<endl;
  if(temp) delete []temp;
    

  cout << "... end reading default parameters"<<endl;
}

void MEMM::calculWeightPollenDonor()
{

  Poids.resize( NPar, 1.0 );

  cout <<endl << "Calculing pollen donor weight..."<<endl;
  for (int p=0; p<NPar; p++)
  {
    for (int k=0; k<Nw; k++)
      Poids[p]*=((*allParents[p])._covarquanti[k]);
    cout << Poids[p] << " | ";
  }
  cout<<endl;
}


/***************************** PRIVATE ***********************************/
